@@ -13,7 +13,6 @@ seconds to allow some time to pass between calls to check the modify
 time on files.
 """
 
-import io
 import os
 import sys
 import shutil
@@ -31,6 +30,7 @@ import contextlib
 import stat
 
 import pytest
+from more_itertools import ilen
 
 import path
 from path import Path
@@ -79,6 +79,12 @@ class TestBasics:
             d = Path('D:\\')
             assert d.relpathto(boz) == boz
 
+    def test_construction_without_args(self):
+        """
+        Path class will construct a path to current directory when called with no arguments.
+        """
+        assert Path() == '.'
+
     def test_construction_from_none(self):
         """ """
         with pytest.raises(TypeError):
@@ -94,7 +100,7 @@ class TestBasics:
         """Test compatibility with ordinary strings."""
         x = Path('xyzzy')
         assert x == 'xyzzy'
-        assert x == str('xyzzy')
+        assert x == 'xyzzy'
 
         # sorting
         items = [Path('fhj'), Path('fgh'), 'E', Path('d'), 'A', Path('B'), 'c']
@@ -125,9 +131,9 @@ class TestBasics:
         assert f.name == 'xyzzy.py'
         assert f.parent.name == os_choose(nt='Lib', posix='lib')
 
-        # .ext
-        assert f.ext == '.py'
-        assert f.parent.ext == ''
+        # .suffix
+        assert f.suffix == '.py'
+        assert f.parent.suffix == ''
 
         # .drive
         assert f.drive == os_choose(nt='C:', posix='')
@@ -301,12 +307,15 @@ class TestBasics:
 
 
 class TestReadWriteText:
-    @pytest.mark.filterwarnings('ignore:Writing bytes in write_text')
     def test_read_write(self, tmpdir):
         file = path.Path(tmpdir) / 'filename'
-        file.write_text('hello world')
-        assert file.read_text() == 'hello world'
+        file.write_text('hello world', encoding='utf-8')
+        assert file.read_text(encoding='utf-8') == 'hello world'
         assert file.read_bytes() == b'hello world'
+
+    @pytest.mark.filterwarnings('ignore:Writing bytes in write_text')
+    def test_write_text_bytes(self, tmpdir):
+        file = path.Path(tmpdir) / 'filename'
         file.write_text(b'hello world')
 
 
@@ -316,7 +325,7 @@ class TestPerformance:
         args = [sys.executable, '-m', 'timeit', '-n', '1', '-r', '1', '-u', 'usec'] + [
             cmd
         ]
-        res = subprocess.check_output(args, universal_newlines=True)
+        res = subprocess.check_output(args, text=True, encoding='utf-8')
         dur = re.search(r'(\d+) usec per loop', res).group(1)
         return datetime.timedelta(microseconds=int(dur))
 
@@ -344,7 +353,7 @@ class TestLinks:
         target = Path(tmpdir) / 'target'
         target.write_text('hello', encoding='utf-8')
         link = target.link(Path(tmpdir) / 'link')
-        assert link.read_text() == 'hello'
+        assert link.read_text(encoding='utf-8') == 'hello'
 
     def test_symlink_none(self, tmpdir):
         root = Path(tmpdir)
@@ -422,7 +431,7 @@ def test_chroot(monkeypatch):
     results = []
     monkeypatch.setattr(os, 'chroot', results.append)
     Path().chroot()
-    assert results == ['']
+    assert results == [Path()]
 
 
 @pytest.mark.skipif("not hasattr(Path, 'startfile')")
@@ -430,7 +439,7 @@ def test_startfile(monkeypatch):
     results = []
     monkeypatch.setattr(os, 'startfile', results.append)
     Path().startfile()
-    assert results == ['']
+    assert results == [Path()]
 
 
 class TestScratchDir:
@@ -473,9 +482,8 @@ class TestScratchDir:
             assert t0 <= ct <= t1
 
         time.sleep(threshold * 2)
-        fobj = open(f, 'ab')
-        fobj.write('some bytes'.encode('utf-8'))
-        fobj.close()
+        with open(f, 'ab') as fobj:
+            fobj.write(b'some bytes')
 
         time.sleep(threshold * 2)
         t2 = time.time() - threshold
@@ -505,7 +513,7 @@ class TestScratchDir:
 
     def test_listing(self, tmpdir):
         d = Path(tmpdir)
-        assert d.listdir() == []
+        assert list(d.iterdir()) == []
 
         f = 'testfile.txt'
         af = d / f
@@ -514,7 +522,7 @@ class TestScratchDir:
         try:
             assert af.exists()
 
-            assert d.listdir() == [af]
+            assert list(d.iterdir()) == [af]
 
             # .glob()
             assert d.glob('testfile.txt') == [af]
@@ -535,11 +543,10 @@ class TestScratchDir:
         # Try a test with 20 files
         files = [d / ('%d.txt' % i) for i in range(20)]
         for f in files:
-            fobj = open(f, 'w')
-            fobj.write('some text\n')
-            fobj.close()
+            with open(f, 'w', encoding='utf-8') as fobj:
+                fobj.write('some text\n')
         try:
-            files2 = d.listdir()
+            files2 = list(d.iterdir())
             files.sort()
             files2.sort()
             assert files == files2
@@ -550,7 +557,7 @@ class TestScratchDir:
 
     @pytest.fixture
     def bytes_filename(self, tmpdir):
-        name = r'r\xe9\xf1emi'.encode('latin-1')
+        name = br'r\xe9\xf1emi'
         base = str(tmpdir).encode('ascii')
         try:
             with open(os.path.join(base, name), 'wb'):
@@ -559,17 +566,17 @@ class TestScratchDir:
             raise pytest.skip(f"Invalid encodings disallowed {exc}")
         return name
 
-    def test_listdir_other_encoding(self, tmpdir, bytes_filename):  # pragma: nocover
+    def test_iterdir_other_encoding(self, tmpdir, bytes_filename):  # pragma: nocover
         """
         Some filesystems allow non-character sequences in path names.
-        ``.listdir`` should still function in this case.
+        ``.iterdir`` should still function in this case.
         See issue #61 for details.
         """
         # first demonstrate that os.listdir works
         assert os.listdir(str(tmpdir).encode('ascii'))
 
         # now try with path
-        results = Path(tmpdir).listdir()
+        results = Path(tmpdir).iterdir()
         (res,) = results
         assert isinstance(res, Path)
         assert len(res.basename()) == len(bytes_filename)
@@ -636,9 +643,8 @@ class TestScratchDir:
         testA.mkdir()
         testB.mkdir()
 
-        f = open(testFile, 'w')
-        f.write('x' * 10000)
-        f.close()
+        with open(testFile, 'w', encoding='utf-8') as f:
+            f.write('x' * 10000)
 
         # Test simple file copying.
         testFile.copyfile(testCopy)
@@ -658,7 +664,7 @@ class TestScratchDir:
         testA.copytree(testC)
         assert testC.isdir()
         self.assertSetsEqual(
-            testC.listdir(),
+            testC.iterdir(),
             [testC / testCopy.name, testC / testFile.name, testCopyOfLink],
         )
         assert not testCopyOfLink.islink()
@@ -671,7 +677,7 @@ class TestScratchDir:
         testA.copytree(testC, True)
         assert testC.isdir()
         self.assertSetsEqual(
-            testC.listdir(),
+            testC.iterdir(),
             [testC / testCopy.name, testC / testFile.name, testCopyOfLink],
         )
         if hasattr(os, 'symlink'):
@@ -681,7 +687,7 @@ class TestScratchDir:
         # Clean up.
         testDir.rmtree()
         assert not testDir.exists()
-        self.assertList(d.listdir(), [])
+        self.assertList(d.iterdir(), [])
 
     def assertList(self, listing, expected):
         assert sorted(listing) == sorted(expected)
@@ -697,7 +703,7 @@ class TestScratchDir:
 
             for name in names:
                 (e / name).touch()
-        self.assertList(d.listdir('*.tmp'), [d / 'x.tmp', d / 'xdir.tmp'])
+        self.assertList(d.iterdir('*.tmp'), [d / 'x.tmp', d / 'xdir.tmp'])
         self.assertList(d.files('*.tmp'), [d / 'x.tmp'])
         self.assertList(d.dirs('*.tmp'), [d / 'xdir.tmp'])
         self.assertList(
@@ -739,7 +745,7 @@ class TestScratchDir:
         stripped = [line.replace('\n', '') for line in expectedLines]
 
         # write bytes manually to file
-        with io.open(p, 'wb') as strm:
+        with open(p, 'wb') as strm:
             strm.write(given.encode(encoding))
 
         # test all 3 path read-fully functions, including
@@ -820,8 +826,8 @@ class TestScratchDir:
         p = (TempDir() / 'test.txt').touch()
         txt = "0123456789"
         size = 5
-        p.write_text(txt)
-        for i, chunk in enumerate(p.chunks(size)):
+        p.write_text(txt, encoding='utf-8')
+        for i, chunk in enumerate(p.chunks(size, encoding='utf-8')):
             assert chunk == txt[i * size : i * size + size]
 
         assert i == len(txt) / size - 1
@@ -900,7 +906,7 @@ class TestMergeTree:
         self.subdir_a.mkdir()
         self.subdir_b.mkdir()
 
-        with open(self.test_file, 'w') as f:
+        with open(self.test_file, 'w', encoding='utf-8') as f:
             f.write('x' * 10000)
 
         self.test_file.symlink(self.test_link)
@@ -913,19 +919,21 @@ class TestMergeTree:
     def test_with_nonexisting_dst_kwargs(self):
         self.subdir_a.merge_tree(self.subdir_b, symlinks=True)
         assert self.subdir_b.isdir()
-        expected = set(
-            (self.subdir_b / self.test_file.name, self.subdir_b / self.test_link.name)
-        )
-        assert set(self.subdir_b.listdir()) == expected
+        expected = {
+            self.subdir_b / self.test_file.name,
+            self.subdir_b / self.test_link.name,
+        }
+        assert set(self.subdir_b.iterdir()) == expected
         self.check_link()
 
     def test_with_nonexisting_dst_args(self):
         self.subdir_a.merge_tree(self.subdir_b, True)
         assert self.subdir_b.isdir()
-        expected = set(
-            (self.subdir_b / self.test_file.name, self.subdir_b / self.test_link.name)
-        )
-        assert set(self.subdir_b.listdir()) == expected
+        expected = {
+            self.subdir_b / self.test_file.name,
+            self.subdir_b / self.test_link.name,
+        }
+        assert set(self.subdir_b.iterdir()) == expected
         self.check_link()
 
     def test_with_existing_dst(self):
@@ -935,20 +943,18 @@ class TestMergeTree:
         self.test_link.remove()
         test_new = self.subdir_a / 'newfile.txt'
         test_new.touch()
-        with open(self.test_file, 'w') as f:
+        with open(self.test_file, 'w', encoding='utf-8') as f:
             f.write('x' * 5000)
 
         self.subdir_a.merge_tree(self.subdir_b, True)
 
         assert self.subdir_b.isdir()
-        expected = set(
-            (
-                self.subdir_b / self.test_file.name,
-                self.subdir_b / self.test_link.name,
-                self.subdir_b / test_new.name,
-            )
-        )
-        assert set(self.subdir_b.listdir()) == expected
+        expected = {
+            self.subdir_b / self.test_file.name,
+            self.subdir_b / self.test_link.name,
+            self.subdir_b / test_new.name,
+        }
+        assert set(self.subdir_b.iterdir()) == expected
         self.check_link()
         assert len(Path(self.subdir_b / self.test_file.name).bytes()) == 5000
 
@@ -960,7 +966,7 @@ class TestMergeTree:
         self.subdir_a.merge_tree(self.subdir_b, ignore=ignore)
 
         assert self.subdir_b.isdir()
-        assert self.subdir_b.listdir() == [self.subdir_b / self.test_file.name]
+        assert list(self.subdir_b.iterdir()) == [self.subdir_b / self.test_file.name]
 
     def test_only_newer(self):
         """
@@ -969,11 +975,20 @@ class TestMergeTree:
         newer copies in the dest.
         """
         target = self.subdir_b / 'testfile.txt'
-        target.write_text('this is newer')
+        target.write_text('this is newer', encoding='utf-8')
         self.subdir_a.merge_tree(
             self.subdir_b, copy_function=path.only_newer(shutil.copy2)
         )
-        assert target.read_text() == 'this is newer'
+        assert target.read_text(encoding='utf-8') == 'this is newer'
+
+    def test_nested(self):
+        self.subdir_a.joinpath('subsub').mkdir()
+        self.subdir_a.merge_tree(self.subdir_b)
+        assert self.subdir_b.joinpath('subsub').isdir()
+
+    def test_listdir(self):
+        with pytest.deprecated_call():
+            Path().listdir()
 
 
 class TestChdir:
@@ -1101,22 +1116,22 @@ class TestPatternMatching:
         assert p.fnmatch('foobar', normcase=normcase)
         assert p.fnmatch('FOO[ABC]AR', normcase=normcase)
 
-    def test_listdir_simple(self):
+    def test_iterdir_simple(self):
         p = Path('.')
-        assert len(p.listdir()) == len(os.listdir('.'))
+        assert ilen(p.iterdir()) == len(os.listdir('.'))
 
-    def test_listdir_empty_pattern(self):
+    def test_iterdir_empty_pattern(self):
         p = Path('.')
-        assert p.listdir('') == []
+        assert list(p.iterdir('')) == []
 
-    def test_listdir_patterns(self, tmpdir):
+    def test_iterdir_patterns(self, tmpdir):
         p = Path(tmpdir)
         (p / 'sub').mkdir()
         (p / 'File').touch()
-        assert p.listdir('s*') == [p / 'sub']
-        assert len(p.listdir('*')) == 2
+        assert list(p.iterdir('s*')) == [p / 'sub']
+        assert ilen(p.iterdir('*')) == 2
 
-    def test_listdir_custom_module(self, tmpdir):
+    def test_iterdir_custom_module(self, tmpdir):
         """
         Listdir patterns should honor the case sensitivity of the path module
         used by that Path class.
@@ -1125,14 +1140,14 @@ class TestPatternMatching:
         p = always_unix(tmpdir)
         (p / 'sub').mkdir()
         (p / 'File').touch()
-        assert p.listdir('S*') == []
+        assert list(p.iterdir('S*')) == []
 
         always_win = Path.using_module(ntpath)
         p = always_win(tmpdir)
-        assert p.listdir('S*') == [p / 'sub']
-        assert p.listdir('f*') == [p / 'File']
+        assert list(p.iterdir('S*')) == [p / 'sub']
+        assert list(p.iterdir('f*')) == [p / 'File']
 
-    def test_listdir_case_insensitive(self, tmpdir):
+    def test_iterdir_case_insensitive(self, tmpdir):
         """
         Listdir patterns should honor the case sensitivity of the path module
         used by that Path class.
@@ -1140,8 +1155,8 @@ class TestPatternMatching:
         p = Path(tmpdir)
         (p / 'sub').mkdir()
         (p / 'File').touch()
-        assert p.listdir(matchers.CaseInsensitive('S*')) == [p / 'sub']
-        assert p.listdir(matchers.CaseInsensitive('f*')) == [p / 'File']
+        assert list(p.iterdir(matchers.CaseInsensitive('S*'))) == [p / 'sub']
+        assert list(p.iterdir(matchers.CaseInsensitive('f*'))) == [p / 'File']
         assert p.files(matchers.CaseInsensitive('S*')) == []
         assert p.dirs(matchers.CaseInsensitive('f*')) == []
 
@@ -1185,29 +1200,29 @@ class TestInPlace:
     @classmethod
     def create_reference(cls, tmpdir):
         p = Path(tmpdir) / 'document'
-        with p.open('w') as stream:
+        with p.open('w', encoding='utf-8') as stream:
             stream.write(cls.reference_content)
         return p
 
     def test_line_by_line_rewrite(self, tmpdir):
         doc = self.create_reference(tmpdir)
         # reverse all the text in the document, line by line
-        with doc.in_place() as (reader, writer):
+        with doc.in_place(encoding='utf-8') as (reader, writer):
             for line in reader:
                 r_line = ''.join(reversed(line.strip())) + '\n'
                 writer.write(r_line)
-        with doc.open() as stream:
+        with doc.open(encoding='utf-8') as stream:
             data = stream.read()
         assert data == self.reversed_content
 
     def test_exception_in_context(self, tmpdir):
         doc = self.create_reference(tmpdir)
         with pytest.raises(RuntimeError) as exc:
-            with doc.in_place() as (reader, writer):
+            with doc.in_place(encoding='utf-8') as (reader, writer):
                 writer.write(self.alternate_content)
                 raise RuntimeError("some error")
         assert "some error" in str(exc.value)
-        with doc.open() as stream:
+        with doc.open(encoding='utf-8') as stream:
             data = stream.read()
         assert 'Lorem' not in data
         assert 'lazy dog' in data
